<?php 


/**
 * @file
 * File RDF Admin settings.
 */


function file_rdf_admin_settings_database() {
  $header = array(t('Name'), t('Database Name'), t('Graphs'), array('data' => t('Operations'), 'colspan' => '2'));
  
  $rows = array();
  foreach (file_rdf_getDBs() as $name => $info) {
  
     $rows[] = array(
       l($info->name, 'admin/settings/file/file_rdf/database/edit/' . $info->module),
       $info->module,
       $info->graphs,
       l(t('edit'), 'admin/settings/file/file_rdf/database/edit/' . $info->module),
       $info->module != 'file_rdf' ? l(t('delete'), 'admin/settings/file/file_rdf/database/delete/' . $info->module) : '',
   );
  }

  return theme('table', $header, $rows);
}


function file_rdf_admin_db_edit($name = '') {

  if (empty($name)) {
    return drupal_get_form('file_rdf_admin_db_form');
  }
  else {
    return drupal_get_form('file_rdf_admin_db_form', file_rdf_getDB($name));
  }

}
function file_rdf_admin_db_delete($form_state, $name) {


  if (user_access('administer ARC2 Databases')) {
    $form['name'] = array('#type' => 'value', '#value' => $name);
    $output = confirm_form($form,
      t('Are you sure you want to delete the Database %title?', array('%title' => $name)),
      isset($_GET['destination']) ? $_GET['destination'] : 'admin/settings/file/file_rdf/database',
      t('This action will destroy all data contained in the Database and cannot be undone.'));
  }
  return $output;
}



function file_rdf_admin_db_delete_submit($form, &$form_state) {
  if ($form_state['values']['confirm']) {
    file_rdf_delete_repository($form_state['values']['name']);
    drupal_set_message(t('The Databse has been deleted.'));

    $form_state['redirect'] = 'admin/settings/file/file_rdf/database';
  }
}


function file_rdf_admin_db_form(&$form_state, $edit = array('name' => '', 'title' => '', 'description' => '')) {
  

  $edit = (object)$edit;
  $form = array();

  $form['identity'] = array('#type' => 'fieldset', '#title' => t('Identification'));

  $form['identity']['name'] = array(
                              '#type' => 'textfield', 
                              '#title' => t('Name'), 
                              '#default_value' => $edit->name, 
                              '#maxlength' => 64, 
                              '#required' => TRUE,
                              '#disabled' => $edit->module == 'file_rdf' ? TRUE : FALSE,
                              '#description' => t('The human-readable name of the Database.')
  );
  $form['identity']['module'] = array(
                              '#type' => 'textfield', 
                              '#title' => t('Modul ID'), 
                              '#default_value' => $edit->module, 
                              '#maxlength' => 32, 
                              '#required' => TRUE,
                              '#disabled' => $edit->module == 'file_rdf' ? TRUE : FALSE,
                              '#description' => t('The machine-readable name of the Database.'));
  
  $form['identity']['graphs'] = array(
                              '#type' => 'textfield', 
                              '#title' => t('DB Graphs'), 
                              '#default_value' => $edit->graphs,
                              '#description' => t('Graphs'));
  
  $form['identity']['description'] = array(
                              '#title' => t('Description'), 
                              '#type' => 'textarea', 
                              '#default_value' => $edit->description, 
                              '#rows' => 2,
                              '#disabled' => $edit->module == 'file_rdf' ? TRUE : FALSE,
                              '#description' => t('A brief description of the Database.')
  );
  if ($edit->module == 'file_rdf') {
    $form['name'] = array('#type' => 'hidden', '#value' => $edit->name);
    $form['module'] = array('#type' => 'hidden', '#value' => $edit->module);
    $form['description'] = array('#type' => 'hidden', '#value' => $edit->description);
  }
  $form['key'] = array('#type' => 'hidden', '#value' => $edit->module);
  $form['submit'] = array(
                              '#type' => 'submit', 
                              '#value' => empty($edit->module) ? t('Create new Database') : t('Update Database'));

  return $form;
}

function file_rdf_admin_db_form_validate($form, &$form_state) {
  extract($form_state['values'], EXTR_SKIP | EXTR_REFS);

  if (!preg_match('/^[a-z]+[a-z\d_]*$/', $module)) {
    form_set_error('module', t('The machine-readable name can only consist of lowercase letters, underscores, and numbers.', array('%name' => $name)));
  }

  if (array_search($module, file_rdf_getDBs()) !== FALSE) { // FIXME
    form_set_error('module', t('The machine-readable name %name is already used by another Database.', array('%name' => $name)));
  }
}

function file_rdf_admin_db_form_submit($form, &$form_state) {
  
  
  extract($form_state['values'], EXTR_SKIP | EXTR_REFS);

  if (empty($key)) {
    file_rdf_create_repository($module, $name, $graphs, array('description' => $description));
    drupal_set_message(t('The Database has been created.'));
  }
  else {
    if ($key != $module) {
      file_rdf_rename_repository($key, $module);
    }
    file_rdf_update_repository($module, $name, $graphs, array('description' => $description));
    drupal_set_message(t('The Database has been updated.'));
  }

  $form_state['redirect'] = 'admin/settings/file/file_rdf/database';
}


function file_rdf_create_repository($module, $name, $graphs, array $options = array()) {

  $store = file_rdf_getStore($module);
  
  if (!$store->isSetUp($config)) {
    $store->setUp();
  }
  
  drupal_write_record('file_rdf_dbs', $record = (object)array(
    'name'    => $name,
    'module'  => $module,
    'graphs'  => $graphs,
    'enabled' => TRUE,
    'options' => serialize($options),
  ));

  //rdf_get_schema(db_escape_table(RDF_DB_TABLE_PREFIX . $name), TRUE); // clear the schema cache
  return $results[0]['success'];
}


function file_rdf_update_repository($module, $name, $graphs, array $options = array()) {
  db_query("UPDATE {file_rdf_dbs} SET options = '%s', name = '%s', graphs = '%s' WHERE module = '%s'", serialize($options), $name, $graphs, $module);
}

function file_rdf_rename_repository($old_module, $new_module) {
  
  db_rename_table($results,
  db_escape_table('file_rdf_' . $old_module .'_g2t'),
  db_escape_table('file_rdf_' . $new_module .'_g2t'));
  
  db_rename_table($results,
  db_escape_table('file_rdf_' . $old_module .'_id2val'),
  db_escape_table('file_rdf_' . $new_module .'_id2val'));
  
  db_rename_table($results,
  db_escape_table('file_rdf_' . $old_module .'_o2val'),
  db_escape_table('file_rdf_' . $new_module .'_o2val'));
  
  db_rename_table($results,
  db_escape_table('file_rdf_' . $old_module .'_s2val'),
  db_escape_table('file_rdf_' . $new_module .'_s2val'));
  
  db_rename_table($results,
  db_escape_table('file_rdf_' . $old_module .'_setting'),
  db_escape_table('file_rdf_' . $new_module .'_setting'));
  
  db_rename_table($results,
  db_escape_table('file_rdf_' . $old_module .'_triple'),
  db_escape_table('file_rdf_' . $new_module .'_triple'));
  
  
  db_query("UPDATE {file_rdf_dbs} SET module = '%s' WHERE module = '%s'", $new_module, $old_module);
  return $results[0]['success'];
}

function file_rdf_delete_repository($module) {
  if ($module != 'file_rdf') {
    $store = file_rdf_getStore($module);
    $store->drop();
    if ($errs = $store->getErrors()) {
    
     return FALSE;
    }
    db_query("DELETE FROM {file_rdf_dbs} WHERE module = '%s'", $module);
    return $results[0]['success'];
  }
  else {
    return FALSE;
  }
}